@@ -4,7 +4,7 @@ use super::*;
 use basic_dsp_vector::*;
 use basic_dsp_vector::window_functions::*;
 use basic_dsp_vector::conv_types::*;
-use num::complex::Complex32;
+use num::complex::*;
 use std::slice;
 use std::os::raw::c_void;
 use std::mem;
@@ -193,6 +193,55 @@ pub extern "C" fn complex_sum32(vector: &VecBuf) -> Complex32 {
 pub extern "C" fn complex_sum_sq32(vector: &VecBuf) -> Complex32 {
     vector.vec.sum_sq()
 }
+#[no_mangle]
+pub extern "C" fn real_dot_product_prec32(vector: &VecBuf,
+                                     operand: &VecBuf)
+                                     -> ScalarInteropResult<f32> {
+    vector.convert_scalar(|v| {
+        PreciseDotProductOps::<f32, GenDspVec<Vec<f32>, f32>>::dot_product_prec(v, &operand.vec)
+    },
+    0.0)
+}
+
+#[no_mangle]
+pub extern "C" fn complex_dot_product_prec32(vector: &VecBuf,
+                                        operand: &VecBuf)
+                                        -> ScalarInteropResult<Complex32> {
+    vector.convert_scalar(|v| {
+        PreciseDotProductOps::<Complex32, GenDspVec<Vec<f32>, f32>>::dot_product_prec(v, &operand.vec)
+    },
+    Complex32::new(0.0, 0.0))
+}
+
+#[no_mangle]
+pub extern "C" fn real_statistics_prec32(vector: &VecBuf) -> Statistics<f64> {
+    vector.vec.statistics_prec()
+}
+
+#[no_mangle]
+pub extern "C" fn complex_statistics_prec32(vector: &VecBuf) -> Statistics<Complex64> {
+    vector.vec.statistics_prec()
+}
+
+#[no_mangle]
+pub extern "C" fn real_sum_prec32(vector: &VecBuf) -> f64 {
+    vector.vec.sum_prec()
+}
+
+#[no_mangle]
+pub extern "C" fn real_sum_sq_prec32(vector: &VecBuf) -> f64 {
+    vector.vec.sum_sq_prec()
+}
+
+#[no_mangle]
+pub extern "C" fn complex_sum_prec32(vector: &VecBuf) -> Complex64 {
+    vector.vec.sum_prec()
+}
+
+#[no_mangle]
+pub extern "C" fn complex_sum_sq_prec32(vector: &VecBuf) -> Complex64 {
+    vector.vec.sum_sq_prec()
+}
 
 /// `padding_option` argument is translated to:
 /// Returns the vector domain as integer:
@@ -362,7 +411,7 @@ pub extern "C" fn unwrap32(vector: Box<VecBuf>, value: f32) -> VectorInteropResu
 
 #[no_mangle]
 pub extern "C" fn swap_halves32(vector: Box<VecBuf>) -> VectorInteropResult<VecBuf> {
-    vector.convert_vec(|v, b| Ok(v.swap_halves_b(b)))
+    vector.convert_vec(|v, _| Ok(v.swap_halves()))
 }
 
 #[no_mangle]
@@ -661,13 +710,42 @@ pub extern "C" fn overwrite_data32(mut vector: Box<VecBuf>,
     }
 }
 
+#[no_mangle]
+pub extern "C" fn real_statistics_split32(vector: &VecBuf,
+                                             data: *mut Statistics<f32>,
+                                             len: usize)
+                                             -> i32 {
+    let mut data = unsafe { slice::from_raw_parts_mut(data, len) };
+    let stats = vector.vec.statistics_split(data.len());
+    for i in 0..stats.len() {
+        data[i] = stats[i];
+    }
+
+    0
+}
+
+#[no_mangle]
+pub extern "C" fn complex_statistics_split32(vector: &VecBuf,
+                                                data: *mut Statistics<Complex32>,
+                                                len: usize)
+                                                -> i32 {
+    let mut data = unsafe { slice::from_raw_parts_mut(data, len) };
+    let stats = vector.vec.statistics_split(data.len());
+    for i in 0..stats.len() {
+        data[i] = stats[i];
+    }
+
+    0
+}
+
+
 #[no_mangle]
 pub extern "C" fn real_statistics_splitted32(vector: &VecBuf,
                                              data: *mut Statistics<f32>,
                                              len: usize)
                                              -> i32 {
     let mut data = unsafe { slice::from_raw_parts_mut(data, len) };
-    let stats = vector.vec.statistics_splitted(data.len());
+    let stats = vector.vec.statistics_split(data.len());
     for i in 0..stats.len() {
         data[i] = stats[i];
     }
@@ -681,7 +759,7 @@ pub extern "C" fn complex_statistics_splitted32(vector: &VecBuf,
                                                 len: usize)
                                                 -> i32 {
     let mut data = unsafe { slice::from_raw_parts_mut(data, len) };
-    let stats = vector.vec.statistics_splitted(data.len());
+    let stats = vector.vec.statistics_split(data.len());
     for i in 0..stats.len() {
         data[i] = stats[i];
     }
