@@ -5,7 +5,7 @@ use super::*;
 use basic_dsp_vector::*;
 use basic_dsp_vector::window_functions::*;
 use basic_dsp_vector::conv_types::*;
-use num::complex::Complex64;
+use num::complex::*;
 use std::slice;
 use std::os::raw::c_void;
 use std::mem;
@@ -194,6 +194,55 @@ pub extern "C" fn complex_sum64(vector: &VecBuf) -> Complex64 {
 pub extern "C" fn complex_sum_sq64(vector: &VecBuf) -> Complex64 {
     vector.vec.sum_sq()
 }
+#[no_mangle]
+pub extern "C" fn real_dot_product_prec64(vector: &VecBuf,
+                                     operand: &VecBuf)
+                                     -> ScalarInteropResult<f64> {
+    vector.convert_scalar(|v| {
+        PreciseDotProductOps::<f64, GenDspVec<Vec<f64>, f64>>::dot_product_prec(v, &operand.vec)
+    },
+    0.0)
+}
+
+#[no_mangle]
+pub extern "C" fn complex_dot_product_prec64(vector: &VecBuf,
+                                        operand: &VecBuf)
+                                        -> ScalarInteropResult<Complex64> {
+    vector.convert_scalar(|v| {
+        PreciseDotProductOps::<Complex64, GenDspVec<Vec<f64>, f64>>::dot_product_prec(v, &operand.vec)
+    },
+    Complex64::new(0.0, 0.0))
+}
+
+#[no_mangle]
+pub extern "C" fn real_statistics_prec64(vector: &VecBuf) -> Statistics<f64> {
+    vector.vec.statistics_prec()
+}
+
+#[no_mangle]
+pub extern "C" fn complex_statistics_prec64(vector: &VecBuf) -> Statistics<Complex64> {
+    vector.vec.statistics_prec()
+}
+
+#[no_mangle]
+pub extern "C" fn real_sum_prec64(vector: &VecBuf) -> f64 {
+    vector.vec.sum_prec()
+}
+
+#[no_mangle]
+pub extern "C" fn real_sum_sq_prec64(vector: &VecBuf) -> f64 {
+    vector.vec.sum_sq_prec()
+}
+
+#[no_mangle]
+pub extern "C" fn complex_sum_prec64(vector: &VecBuf) -> Complex64 {
+    vector.vec.sum_prec()
+}
+
+#[no_mangle]
+pub extern "C" fn complex_sum_sq_prec64(vector: &VecBuf) -> Complex64 {
+    vector.vec.sum_sq_prec()
+}
 
 /// `padding_option` argument is translated to:
 /// Returns the vector domain as integer:
@@ -363,7 +412,7 @@ pub extern "C" fn unwrap64(vector: Box<VecBuf>, value: f64) -> VectorInteropResu
 
 #[no_mangle]
 pub extern "C" fn swap_halves64(vector: Box<VecBuf>) -> VectorInteropResult<VecBuf> {
-    vector.convert_vec(|v, b| Ok(v.swap_halves_b(b)))
+    vector.convert_vec(|v, _| Ok(v.swap_halves()))
 }
 
 #[no_mangle]
@@ -662,13 +711,42 @@ pub extern "C" fn overwrite_data64(mut vector: Box<VecBuf>,
     }
 }
 
+#[no_mangle]
+pub extern "C" fn real_statistics_split64(vector: &VecBuf,
+                                             data: *mut Statistics<f64>,
+                                             len: usize)
+                                             -> i32 {
+    let mut data = unsafe { slice::from_raw_parts_mut(data, len) };
+    let stats = vector.vec.statistics_split(data.len());
+    for i in 0..stats.len() {
+        data[i] = stats[i];
+    }
+
+    0
+}
+
+#[no_mangle]
+pub extern "C" fn complex_statistics_split64(vector: &VecBuf,
+                                                data: *mut Statistics<Complex64>,
+                                                len: usize)
+                                                -> i32 {
+    let mut data = unsafe { slice::from_raw_parts_mut(data, len) };
+    let stats = vector.vec.statistics_split(data.len());
+    for i in 0..stats.len() {
+        data[i] = stats[i];
+    }
+
+    0
+}
+
+
 #[no_mangle]
 pub extern "C" fn real_statistics_splitted64(vector: &VecBuf,
                                              data: *mut Statistics<f64>,
                                              len: usize)
                                              -> i32 {
     let mut data = unsafe { slice::from_raw_parts_mut(data, len) };
-    let stats = vector.vec.statistics_splitted(data.len());
+    let stats = vector.vec.statistics_split(data.len());
     for i in 0..stats.len() {
         data[i] = stats[i];
     }
@@ -682,7 +760,7 @@ pub extern "C" fn complex_statistics_splitted64(vector: &VecBuf,
                                                 len: usize)
                                                 -> i32 {
     let mut data = unsafe { slice::from_raw_parts_mut(data, len) };
-    let stats = vector.vec.statistics_splitted(data.len());
+    let stats = vector.vec.statistics_split(data.len());
     for i in 0..stats.len() {
         data[i] = stats[i];
     }
